////////////////////////////////////////////////////////////////////////////////////
// VoicePackDX - Voice Pack Management and Implementation Module v1.0
//
// Purpose: To allow a more dynamic system for adding custom sounds to
//          TracerDX Chat menu and system Chat Menu than existing modules.
//
// Author:  Bryan Warde -aka-> TracerDX
//
////////////////////////////////////////////////////////////////////////////////////




////////////////////////////////////////////////////////////////////////////////////
// VoicePackDX Initialization
////////////////////////////////////////////////////////////////////////////////////
if(!isObject(VoicePackDX))
    new ScriptObject(VoicePackDX) {

		PackCount = 0;
		Version = "1.0";
		TracerDXChatInstalled = isFile("scripts/autoexec/TracerDXchat.cs");
		ConverterMode = false;
		OldPackCount = -1;
		About = "VoicePackDX is a support scrpit designed allow Voice Pack authors more flexibilty in Voice Pack development and deployment. For more information, point your favorite browser to http://tracerdx.zoneofcontrol.net";

		ChatMenuFunction = "setChatMenu";
		CancelMenuFunction = "cancelChatMenu";
	};

////////////////////////////////////////////////////////////////////////////////////
// Voice Pack Registration/Definition Entry Point Functions
////////////////////////////////////////////////////////////////////////////////////

// %PackName			-	Display Name of package (No String limitation)
// %PackFileObjName		-	ObjectName of package (File Name limitations apply and no spaces)
// %PackKey				-	Default entry point keys
// %PackAuthor			-	Author of pack for credits displat
// %PackDescription		-	Description of Pack for credits display
// %PackVersion			-	Version of this pack.
function RegisterVoicePackage( %PackName, %PackFileObjName, %PackKeys, %PackAuthor, %PackDescription, %PackVersion ) {

	//Register all data members into our DB
	VoicePackDX.PackDisplayName[VoicePackDX.PackCount] = %PackName;
	VoicePackDX.PackObjName[VoicePackDX.PackCount] = %PackFileObjName;
	VoicePackDX.PackAuthor[VoicePackDX.PackCount] = %PackAuthor;
	VoicePackDX.PackDefaultKeys[VoicePackDX.PackCount] = %PackKeys;
	VoicePackDX.PackDescription[VoicePackDX.PackCount] = %PackDescription;
	VoicePackDX.PackVersion[VoicePackDX.PackCount] = %PackVersion;
	
	//Console Registration Message
	echo( "\c4Voice Package #" @ VoicePackDX.PackCount @ ": " @ VoicePackDX.PackDisplayName[VoicePackDX.PackCount] @ " v" @ VoicePackDX.PackVersion[VoicePackDX.PackCount] @ " - [" @ VoicePackDX.PackObjName[VoicePackDX.PackCount] @ "] by " @ VoicePackDX.PackAuthor[VoicePackDX.PackCount] @ " has been Registered Successfully" );

	//Increment for next registration
	VoicePackDX.PackCount++;

}

//Menu Build Functions - Start a new submenu
// %heading		- "X: Heading" where X = key to bind menu to in current level.
function startVPDXMenu(%heading)
{
   %key = getSubStr( %heading, 0, strpos(%heading, ":") );
   %text = getSubStr(%heading, (strpos(%heading, ":")+2), (strlen(%heading) - (strpos(%heading, "-")+2)));

   %menu = new ActionMap();
   ChatMenuList.add( %menu );
   %cm = $CurrentVPDXMenu;
   %cm.bindCmd(keyboard, %key, VoicePackDX.ChatMenuFunction @ "(\"" @ %text @ "\", " @ %menu @ ");", "");
   %cm.option[%cm.optionCount] = %key @ ": " @ %text;
   %cm.command[%cm.optionCount] = %menu;  // Save this off here for later...
   %cm.isMenu[%cm.optionCount] = 1;
   %cm.isVoicePack[%cm.optionCount] = true;		//This is a little value we use to figure out if this bind was added by a voicepack or not
   %cm.optionCount++;
   %menu.parent = %cm;
   %menu.bindCmd(keyboard, escape, VoicePackDX.CancelMenuFunction @ "();", "");
   %menu.optionCount = 0;
   $CurrentVPDXMenu = %menu;
}

//Menu Build Functions - end current menu
function endVPDXMenu() {
	$CurrentVPDXMenu = $CurrentVPDXMenu.parent;
}

//Item Build Function
// %keyDesc		- "X: Item Lable" where X = key to bind item to in current level
// %text		- Text to say for this bind
// %sound		- Sound to play for this bind
// %isTeamChat	- true/false for team chat item
function addVPDXItem(%keyDesc, %text, %sound, %isTeamChat)
{

	//Parse Data Items from %heading string
	%key = getSubStr( %keyDesc, 0, strpos(%keyDesc, ":") );
	%menutext = getSubStr(%keyDesc, (strpos(%keyDesc, ":")+2), (strlen(%keyDesc) - (strpos(%keyDesc, ":")+2)));
	%command = %text @ "~wK" @ %sound;

	//From Previous menu build, grab the current menu stored in global $CurrentVPDXMenu
	%currMenu = $CurrentVPDXMenu;

	//Add a keybind for this chat command
	%currMenu.bindCmd(keyboard, %key, "issueVPDXCmd(\"" @ %command @ "\", " @ %isTeamChat @ ");", "");
	%currMenu.option[%currMenu.optionCount] = %keyDesc;
	%currMenu.command[%currMenu.optionCount] = %command;
	%currMenu.isMenu[%currMenu.optionCount] = 0;
    %currMenu.isVoicePack[%currMenu.optionCount] = true;		//This is a little value we use to figure out if this bind was added by a voicepack or not

	%currMenu.optionCount++;

}



//Handles Key Presses for VPDX Items
// %chatCommand		- "text~wsound"
// %isTeamChat		- true/false for team chat item
function issueVPDXCmd( %chatCommand, %isTeamChat ) {

	echo( %chatCommand );

	if (%isTeamChat)
		commandToServer( 'teammessageSent', %chatCommand );
	else
		commandToServer( 'messageSent', %chatCommand);

	//Clear the Root Chat menu
	cancelChatMenu();

	//Clear the TDXC Menu if it exists also
	if (VoicePackDX.TracerDXChatInstalled)
		cancelTDXCMenu();


}

////////////////////////////////////////////////////////////////////////////////////
// Voice Pack Menu Build Functions
////////////////////////////////////////////////////////////////////////////////////

// Rebuilds the root menu
function RebuildVPDXMenus() {

	//Load User Defined Keys from $pref
	VPDXLoadKeys();

	//Display Starting Message...
	echo("\c4Rebuilding VoicePack menus...");
	
	//Build Base Menu First using a System function...
	activateChatMenu("scripts/voiceBinds.cs");
	
	//Reset Redo Counter and save
	$RedoPackCount = 0;
	%LastRedoCount = 0;


	//Do first Pass
	for (%i = 0; %i < VoicePackDX.PackCount; %i++)
		InstallVoicePackDX( %i );


	//Do Retries...
	while (($RedoPackCount != %LastRedoCount) && ($RedoPackCount != 0)) {

		//Store Amount of Redos
		%LastRedoCount = $RedoPackCount;

		//Reset Redo Counter
		$RedoPackCount = 0;

		for (%i = 0; %i < %LastRedoCount; %i++) {

			InstallVoicePackDX( $RedoPack[%i] );

		}

		//Contingency, just in case the algorith leads to infinite loop, after 1000 passes, exit loop
		%VPDXOverFlowCounter++;
		if (%VPDXOverFlowCounter == 1000) {
			error("OVERFLOW Limit of 1000 Passes Reached on Menu Build");
			break;
		}
		echo(%VPDXOverFlowCounter++);


	}

	//Rebuild TDXC Menus, if they exist
	if (isFile("scripts/autoexec/TracerDXChat.cs"))
		activateTDXCMenu("scripts/TDXMenu" @ $TDXCVoice);

	//Display Finishing Console Message
	echo("\c4VoicePackDX Rebuild Completed");
		

}

//Attempts to install a voice package. Logs failures for attempt on next pass
function InstallVoicePackDX( %i ) {

	%keys = VoicePackDX.PackKeys[%i];
	if (%keys $= "")
		%keys = VoicePackDX.PackDefaultKeys[%i];

	%key = getSubStr(%keys, (strlen(%keys) - 1), 1);

	%packIsOk = true;

	//Get the root chat menu so we may traverse it
	%CurrMenu = $RootChatMenu;

	//If Default Keys is longer than 1 char, then it must be bound to a child of the root menu.
	for (%a = 0; %a < (strlen(%keys) - 1); %a++) {

		%thisKey = getSubStr(%keys, %a, 1);

		%bindPathFound = false;

		for (%b = 0; %b < %CurrMenu.optionCount; %b++)

			if (%thisKey $= getSubStr(%CurrMenu.option[%b], 0, 1)) {

				if (%CurrMenu.isMenu[%b] == 1) {

					%CurrMenu = %CurrMenu.command[%b];
					
					%bindPathFound = true;

				} else {

					%packIsOk = false;

					VoicePackDX.PackProblem[%i] = 2; //Problem Code 2 = Tried to bind to a command

					error("Pack Attempted to Bind to a Command, vice a Menu Item. Need new bind path");

					break;

				}

		}

		if (!%bindPathFound && %packIsOk) {

			%packIsOk = false;

			$RedoPack[$RedoPackCount] = %i;

			$RedoPackCount++;

			VoicePackDX.PackProblem[%i] = 1; //Problem Code 1 = Bound to non-existent bind path

			error("Bind Path Specified for pack does not exist. Retry on next Pass");

			break;

		}

		if (!%packIsOk)
			break;

	}

	//Make Sure that the key is available in this menu....
	for (%b = 0; %b < %CurrMenu.optionCount; %b++)
		if (%key $= getSubStr(%CurrMenu.option[%b], 0, 1)) {

			%packIsOk = false;

			VoicePackDX.PackProblem[%i] = 3; //Problem Code 3 = Bind Already Exists!

			error("Pack Attempted to Bind to an existing Command. Please Redefine.");

		}

	
	if (%packIsOk) {
		//Set the global current menu to correct one for binding keys
		$CurrentVPDXMenu = %CurrMenu;

		//Add a Menu at $CurrentVPDXMenu for this package
		startVPDXMenu(%key @ ": " @ VoicePackDX.PackDisplayName[%i]);

		//Execute Key Binding script
		exec("scripts/" @ VoicePackDX.PackObjName[%i] @ ".VoicePack.cs");

		//Finish The package menu
		endVPDXMenu();

		//Set problem flag to false
		VoicePackDX.PackProblem[%i] = false;

		//Display Console Message for this pack
		echo("\c4 + Voice Package: " @ VoicePackDX.PackDisplayName[%i] @ " was bound successfully to " @ %key @ " or [V" @ %keys @ "]");
	} 

}

// Rebuilds the TDXC menu
function RebuildTDXCMenus() {
	
	//Reset Redo Counter and save
	$RedoPackCount = 0;
	%LastRedoCount = 0;

	//Switch to Binding TDXC Menu Traverse Functions
	VoicePackDX.ChatMenuFunction = "setTDXCMenu";
	VoicePackDX.CancelMenuFunction = "cancelTDXCMenu";

	//Do first Pass
	for (%i = 0; %i < VoicePackDX.PackCount; %i++)
		InstallVoicePackToTDXC( %i );


	//Do Retries...
	while (($RedoPackCount != %LastRedoCount) && ($RedoPackCount != 0)) {

		//Store Amount of Redos
		%LastRedoCount = $RedoPackCount;

		//Reset Redo Counter
		$RedoPackCount = 0;

		for (%i = 0; %i < %LastRedoCount; %i++) {

			InstallVoicePackToTDXC( $RedoPack[%i] );

		}

		//Contingency, just in case the algorith leads to infinite loop, after 1000 passes, exit loop
		%VPDXOverFlowCounter++;
		if (%VPDXOverFlowCounter == 1000) {
			error("OVERFLOW Limit of 1000 Passes Reached on Menu Build");
			break;
		}
		echo(%VPDXOverFlowCounter++);


	}

	//Switch Back to Binding Root Menu Traverse Functions
	VoicePackDX.ChatMenuFunction = "setChatMenu";
	VoicePackDX.CancelMenuFunction = "cancelChatMenu";
		

}

//Attempts to install a voice package. Logs failures for attempt on next pass
function InstallVoicePackToTDXC( %i ) {

	%keys = VoicePackDX.PackKeys[%i];
	if (%keys $= "")
		%keys = VoicePackDX.PackDefaultKeys[%i];

	%key = getSubStr(%keys, (strlen(%keys) - 1), 1);

	%packIsOk = true;

	//Get the root chat menu so we may traverse it
	%CurrMenu = $RootTDXCMenu;

	//If Default Keys is longer than 1 char, then it must be bound to a child of the root menu.
	for (%a = 0; %a < (strlen(%keys) - 1); %a++) {

		%thisKey = getSubStr(%keys, %a, 1);

		%bindPathFound = false;

		for (%b = 0; %b < %CurrMenu.optionCount; %b++)

			if (%thisKey $= getSubStr(%CurrMenu.option[%b], 0, 1)) {

				if (%CurrMenu.isMenu[%b] == 1) {

					%CurrMenu = %CurrMenu.command[%b];
					
					%bindPathFound = true;

				} else {

					%packIsOk = false;

					VoicePackDX.PackProblem[%i] = 2; //Problem Code 2 = Tried to bind to a command

					error("Pack Attempted to Bind to a Command, vice a Menu Item. Need new bind path");

					break;

				}

		}

		if (!%bindPathFound && %packIsOk) {

			%packIsOk = false;

			$RedoPack[$RedoPackCount] = %i;

			$RedoPackCount++;

			VoicePackDX.PackProblem[%i] = 1; //Problem Code 1 = Bound to non-existent bind path

			error("Bind Path Specified for pack does not exist. Retry on next Pass");

			break;

		}

		if (!%packIsOk)
			break;

	}

	//Make Sure that the key is available in this menu....
	for (%b = 0; %b < %CurrMenu.optionCount; %b++)
		if (%key $= getSubStr(%CurrMenu.option[%b], 0, 1)) {

			%packIsOk = false;

			VoicePackDX.PackProblem[%i] = 3; //Problem Code 3 = Bind Already Exists!

			error("Pack Attempted to Bind to an existing Command. Please Redefine.");

		}

	
	if (%packIsOk) {
		//Set the global current menu to correct one for binding keys
		$CurrentVPDXMenu = %CurrMenu;

		//Add a Menu at $CurrentVPDXMenu for this package
		startVPDXMenu(%key @ ": " @ VoicePackDX.PackDisplayName[%i]);

		//Execute Key Binding script
		exec("scripts/" @ VoicePackDX.PackObjName[%i] @ ".VoicePack.cs");

		//Finish The package menu
		endVPDXMenu();

		//Set problem flag to false
		VoicePackDX.PackProblem[%i] = false;

		//Display Console Message for this pack
		echo("\c4 + Voice Package: " @ VoicePackDX.PackDisplayName[%i] @ " was bound successfully to " @ %key @ " or [V" @ %keys @ "]");
	} 

}


//Saves Current Key Settings to $pref
function VPDXSaveKeys() {

	$pref::VPDXKeyCount = 0;

	for (%i = 0; %i < VoicePackDX.PackCount; %i++) {

		if (VoicePackDX.PackKeys[%i] !$= "") {
			$pref::VPDXPackKeys[$pref::VPDXKeyCount] = VoicePackDX.PackKeys[%i];
			$pref::VPDXPackKeysName[$pref::VPDXKeyCount] = VoicePackDX.PackObjName[%i];
			echo("Key Sequence: V" @ $pref::VPDXPackKeys[$pref::VPDXKeyCount] @ " - " @ VoicePackDX.PackKeys[%i] @ " saved for pack: " @ $pref::VPDXPackKeysName[$pref::VPDXKeyCount]);
			$pref::VPDXKeyCount++;
		}

	}
	
	echo($pref::VPDXKeyCount @ " Keys Saved");

}

function VPDXLoadKeys() {

	for (%i = 0; %i < $pref::VPDXKeyCount; %i++)
		for (%a = 0; %a < VoicePackDX.PackCount; %a++)
			if (VoicePackDX.PackObjName[%a] $= $pref::VPDXPackKeysName[%i]) {
				VoicePackDX.PackKeys[%a] = $pref::VPDXPackKeys[%i];
				echo("Voice Pack " @ VoicePackDX.PackObjName[%a] @ " keys = " @ $pref::VPDXPackKeys[%i]);
			}

	echo($pref::VPDXKeyCount @ " Keys Loaded");

}

///////////////////////////////////////////////////////////////////////////////////
// GUI Handler Functions
///////////////////////////////////////////////////////////////////////////////////

//Main Setup GUI launched
function VoicePackDXManager::onWake( %this ) {

	RebuildVPDXMenus();

	VPDXPackList.fillList();

}

function VoicePackDXConverter::onWake( %this ) {

	VPDXOldPackList.fillList();

}

function VoicePackDXAbout::onWake( %this ) {

	VPDXAboutText.setText(VoicePackDX.About);

}


//Voice Pack listing needs to be (re)filled
function VPDXPackList::fillList( %this ) {

	%this.clear();
	for ( %i = 0; %i < VoicePackDX.PackCount; %i++ ) {

		%keys = VoicePackDX.PackKeys[%i];
		if (%keys $= "")
			%keys = VoicePackDX.PackDefaultKeys[%i];

		%keys = "V" @ %keys;

		if (VoicePackDX.PackProblem[%i])
			%color = "\c2";
		else
			%color = "\c1";

		%this.addRow( %i, ("\c5[" @ %keys @ "] " @ %color @ VoicePackDX.PackDisplayName[%i] @ " v" @ VoicePackDX.PackVersion[%i] @ " by " @ VoicePackDX.PackAuthor[%i] ) );

	}

}

//Voice Pack listing needs to be (re)filled
function VPDXOldPackList::fillList( %this ) {

	%this.clear();
	for ( %i = 0; %i <= VoicePackDX.OldPackCount; %i++ ) {

		%this.addRow( %i, (VoicePackDX.OldPack[%i] @ " Voice Pack"));

	}

}


//Info Page GUI Launched
function VoicePackDXInfoPage::onWake( %this ) {
	
	%InfoPageNum = VPDXPackList.getSelectedId();
	$VPDXInfoPageNum = %InfoPageNum;
	VPDXGUIPackName.setText(VoicePackDX.PackDisplayName[%InfoPageNum]);
	VPDXGUIAuthor.setText(VoicePackDX.PackAuthor[%InfoPageNum]);
	VPDXGUIVersion.setText(VoicePackDX.PackVersion[%InfoPageNum]);
	VPDXGUIDescription.setText(VoicePackDX.PackDescription[%InfoPageNum]);

	%keys = VoicePackDX.PackKeys[%InfoPageNum];
	if (%keys $= "")
		%keys = VoicePackDX.PackDefaultKeys[%InfoPageNum];

	VPDXPackKeys.setText(%keys);

	if (VoicePackDX.PackProblem[%InfoPageNum] == 1)
		VPDXGUIProblem.setText("Tried to bind to non-existent bind path. Choose another above.");
	else if (VoicePackDX.PackProblem[%InfoPageNum] == 2)
		VPDXGUIProblem.setText("Tried to bind menu to existing command. Choose another above.");
	else if (VoicePackDX.PackProblem[%InfoPageNum] == 3)
		VPDXGUIProblem.setText("Bind Problem: Key Bind Sequence in use! Choose another above.");
	else
		VPDXGUIProblem.setText("");

}

function VPDXUpdatePackKeys() {

	VoicePackDX.PackKeys[$VPDXInfoPageNum] = $VPDXKeys;

	VPDXSaveKeys();

	RebuildVPDXMenus();
	VPDXPackList.fillList();
	Canvas.popDialog( VoicePackDXInfoPage );

}


function VPDXConvertSelected() {

	%packName = VoicePackDX.OldPack[VPDXOldPackList.getSelectedId()];

	if (%packName !$= "") {

		//Store Current Pack Identifier
		VoicePackDX.ConverterPackID = %packName;

		//Do Conversion
		VPDXConvertPackage( "scripts/" @ %packName @ "VoicePack.cs" , %packName );

		VPDXPackList.fillList();
	}
	

}
////////////////////////////////////////////////////////////////////////////////////
// Batch VoicePack Converter
////////////////////////////////////////////////////////////////////////////////////

function VPDXConvertPackage( %oldPackageFile, %packFileIdentifier ) {

	//Popup GUI "Converting..."
	Canvas.pushDialog(VoicePackDXPopup);

	echo("Converting...");
	//Convert Definition File...

	//Initialize Indent Var
	$VPDXIndent = "";

	//Reset Number appending value
	VoicePackDX.ConverterPackCount = 0;

	//Flip to conversion mode:
	VoicePackDX.ConverterMode = 1;   //Mode 1 = Convert Files

	//Execute The VoicePack Definition file. My Over-ride functions will intercept the commands and write them in VPDX
	//format to the defFile fileObject
	exec(%oldPackageFile);

	//Flip out of conversion mode:
	VoicePackDX.ConverterMode = false;

	//Rebuild Menus with newly converted stuff.
	RebuildVPDXMenus();

	//Popoff GUI "Converting..."
	Canvas.popDialog(VoicePackDXPopup);

	//Display Restart Message
	MessageBoxYesNo("RESTART", "You will need to restart Tribes 2 to use newly converted Voice Packs. Would you like to quit now?", "quit();", "");

}

function writeVPDXDEFFile( %fileID ) {

	if (isObject(defFile)) {
		defFile.close();
		defFile.delete();
	}

	%defFile = "scripts/" @ %fileID @ ".VoicePack.cs";

	new fileObject( "defFile" );
	defFile.openForWrite( %defFile );

	//Write Definition File Header
	defFile.writeLine( "//----------------------------------------------------------------------------\\\\" );
	defFile.writeLine( "//                                                                            \\\\" );
	defFile.writeLine( "//               VoicePackDX - Voice Package Definition File                  \\\\" );
	defFile.writeLine( "//                                                                            \\\\" );
	defFile.writeLine( "//                This file is a conversion written by VPDX                   \\\\" );
	defFile.writeLine( "//                           Conversion Utility                               \\\\" );
	defFile.writeLine( "//                    http://tracerdx.zoneofcontrol.net                       \\\\" );
	defFile.writeLine( "//                                                                            \\\\" );
	defFile.writeLine( "//----------------------------------------------------------------------------\\\\" );
	defFile.writeLine( " " );

}

function writeVPDXINIFile( %fileID, %packDisplayName, %keys ) {

	%iniFile = "scripts/autoexec/" @ %fileID @ ".AutoExec.cs";

	new fileObject( "iniFile" );
	iniFile.openForWrite( %iniFile );

	//Write Definition File Header
	iniFile.writeLine( "//----------------------------------------------------------------------------\\\\" );
	iniFile.writeLine( "//                                                                            \\\\" );
	iniFile.writeLine( "//              VoicePackDX - Voice Package Registration File                 \\\\" );
	iniFile.writeLine( "//                                                                            \\\\" );
	iniFile.writeLine( "//                This file is a conversion written by VPDX                   \\\\" );
	iniFile.writeLine( "//                           Conversion Utility                               \\\\" );
	iniFile.writeLine( "//                    http://tracerdx.zoneofcontrol.net                       \\\\" );
	iniFile.writeLine( "//                                                                            \\\\" );
	iniFile.writeLine( "//----------------------------------------------------------------------------\\\\" );
	iniFile.writeLine( " " );

	//Default Data items that can't be extracted from the old package, but are needed for VPDX packs
	%author = "VPDX Converter";
	%version = VoicePackDX.Version;
	%desc = "This pack is a conversion by VoicePackDX Converter v" @ VoicePackDX.Version;
	
	iniFile.writeLine( "deactivatePackage(" @ %fileID @ "VoicePackageDX);");
	iniFile.writeLine( "package " @ %fileID @ "VoicePackageDX {");
	iniFile.writeLine( " ");
	iniFile.writeLine( "	function DispatchLaunchMode()");
	iniFile.writeLine( "	{");
	iniFile.writeLine( " ");

	iniFile.writeLine( "		RegisterVoicePackage( \"" @ %packDisplayName @ "\", \"" @ %fileID @ "\", \"" @ %keys @ "\", \"" @ %author @ "\", \"" @ %desc @ "\", \"" @ %version @ "\" );");

	iniFile.writeLine( " ");
	iniFile.writeLine( "		parent::DispatchLaunchMode();");
	iniFile.writeLine( " ");
	iniFile.writeLine( "	}");
	iniFile.writeLine( " ");
	iniFile.writeLine( "};");
	iniFile.writeLine( "activatePackage(" @ %fileID @ "VoicePackageDX);");

	iniFile.close();
	iniFile.delete();

	//Now Load in newly converted pack (Just better to make user restart)
	//RegisterVoicePackage( %packDisplayName, %packFileIdentifier, %keys, %author, %desc, %version );

}

function VPDXFixEscapeSequences(%string) {

	%newString = "";

	for (%i = 0; %i < strlen(%string); %i++) {

		%ch = getSubStr(%string, %i, 1);
		switch$ (%ch) {
		case "\"":
			%ch = "\\\"";
		case "\\":
			%ch = "\\\\";
		case "\c0":
			%ch = "\\c0";
		case "\c1":
			%ch = "\\c1";
		case "\c2":
			%ch = "\\c2";
		case "\c3":
			%ch = "\\c3";
		case "\c4":
			%ch = "\\c4";
		case "\c5":
			%ch = "\\c5";
		case "\c6":
			%ch = "\\c6";
		case "\c7":
			%ch = "\\c7";
		case "\c8":
			%ch = "\\c8";
		case "\c9":
			%ch = "\\c9";
		case "\cr":
			%ch = "\\cr";
		}

		%newString = %newString @ %ch;
		
	}

	return %newString;

}

deactivatePackage(VoicePackDXConverter);
package VoicePackDXConverter {

	function startChatMenu( %heading ) {

		if (!VoicePackDX.ConverterMode) {
			parent::startChatMenu( %heading );
			return;
		}

		if (VoicePackDX.ConverterMode == 1) {
		
			//If this is the first startChatMenu(), then it is the start of a "PACK" for VoicePackDX
			if ($VPDXIndent $= "") {

				VoicePackDX.ConverterPackCount++;
				VoicePackDX.ConverterThisPackName = VoicePackDX.ConverterPackID @ VoicePackDX.ConverterPackCount;
				VoicePackDX.ConverterPackKeys = firstWord(%heading);
				VoicePackDX.ConverterPackDisplayName = restWords(%heading);

				writeVPDXDEFFile( VoicePackDX.ConverterThisPackName );

			} else {

				%key = firstWord(%heading);
				%text = restWords(%heading);

				//Some characters, namely Quotes, fuck things up since their escape sequences are lost... this adds it back on
				%text = VPDXFixEscapeSequences(%text);

				echo("\n" @ $VPDXIndent @ "startVPDXMenu( \"" @ %key @ ": " @ %text @ "\" );");

				
				defFile.writeLine( $VPDXIndent );
				defFile.writeLine( $VPDXIndent @ "startVPDXMenu( \"" @ %key @ ": " @ %text @ "\" );" );

				

			}

			$VPDXIndent = $VPDXIndent @ "\t";

		}

	}

	function addChat( %heading, %CannedChatItem ) {

		if (!VoicePackDX.ConverterMode) {
			parent::addChat( %heading, %CannedChatItem );
			return;
		}

		if (VoicePackDX.ConverterMode == 1) {

			%key = firstWord(%heading);
			%htext = restWords(%heading);
			%text = $VPDXChatTable::Text[%CannedChatItem];
			%sound = $VPDXChatTable::Sound[%CannedChatItem];

			//Some characters, namely Quotes, fuck things up since their escape sequences are lost... this adds it back on
			%text = VPDXFixEscapeSequences(%text);
			%htext = VPDXFixEscapeSequences(%htext);
			
			if ($VPDXChatTable::IsTeamChat[%CannedChatItem])
				%isTeamChat = "true";
			else
				%isTeamChat = "false";

			//Truncate K in front of sound
			%sound = getSubStr(%sound, 1, strlen(%sound));

			echo($VPDXIndent @ "addVPDXItem( \"" @ %key @ ": " @ %htext @ "\", \"" @ %text @ "\", \"" @ %sound @ "\", " @ %isTeamChat @ ")");

			//addVPDXItem(%keyDesc, %text, %sound, %isTeamChat)
			defFile.writeLine( $VPDXIndent @ "addVPDXItem( \"" @ %key @ ": " @ %htext @ "\", \"" @ %text @ "\", \"" @ %sound @ "\", " @ %isTeamChat @ ");" );

		}

	}

	function endChatMenu() {

		if (!VoicePackDX.ConverterMode) {
			echo("SKIPPED");
			parent::endChatMenu();
			return;
		}
		
		if (VoicePackDX.ConverterMode == 1) {

			$VPDXIndent = getSubStr($VPDXIndent, 0, (strlen($VPDXIndent) - 1));

			if ($VPDXIndent $= "") {

				writeVPDXINIFile( VoicePackDX.ConverterThisPackName, VoicePackDX.ConverterPackDisplayName, VoicePackDX.ConverterPackKeys );

			} else {

				echo($VPDXIndent @ "endVPDXMenu();");

				defFile.writeLine( $VPDXIndent @ "endVPDXMenu();" );
				defFile.writeLine( $VPDXIndent );

			}

		}
		

	}

	function installChatItem( %command, %text, %audioFile, %animCel, %teamOnly, %defaultkeys, %play3D ) {

		if (!VoicePackDX.ConverterMode) {
			parent::installChatItem( %command, %text, %audioFile, %animCel, %teamOnly, %defaultkeys, %play3D );
			return;
		}

		if (VoicePackDX.ConverterMode == 1) {
			$VPDXChatTable::Text[%command] = %text;
			$VPDXChatTable::Sound[%command] = %audioFile;
			$VPDXChatTable::IsTeamChat[%command] = %teamOnly;
		}

	}


	//This is where I intercept old VoicePack registration attempts. This effectively disables the old VoicePack.vl2
	//I also use the intercepted information to try and convert the old voicepack.
	function VoicePackRegisterName( %packName ) {

		VPDXCheckOldPack( %packName );

	}

};
activatePackage(VoicePackDXConverter);

function VPDXCheckOldPack( %packName ) {

	VoicePackDX.OldPackCount++;
	VoicePackDX.OldPack[VoicePackDX.OldPackCount] = %packName;

}

////////////////////////////////////////////////////////////////////////////////////
// Over-ride functions
////////////////////////////////////////////////////////////////////////////////////
deactivatePackage(VoicePackDX);
package VoicePackDX {


	//This function intercepts incoming chat bind messages, we will need to grap voicepack messages here.
	function clientCmdChatMessage(%sender, %voice, %pitch, %msgString, %a1, %a2, %a3, %a4, %a5, %a6, %a7, %a8, %a9, %a10)
	{
		%message = detag( %msgString ); $MaxMessageWavLength = 120000; //120000 = 2 minutes
		if( (%message $= "" ) || isClientChatMuted(%sender) )
			return;
		%wavStart = strstr(%message,"~w");
		if ( %wavStart == -1 )
			addMessageHudLine(%message);
		else
			if(strstr(%message, "~wK") !=-1) {
				%voice = "custom"; //Assign our base directory here.
				%wav = getSubStr(%message, %wavStart + 3, 1000); //Move the name three spaces for our delim
				%wavFile = "voice/" @ %voice @ "/" @ %wav @ ".wav";
				if (%pitch < 0.5 || %pitch > 2.0)
					%pitch = 1.0;
				%wavLengthMS = alxGetWaveLen(%wavFile) * %pitch;
				if ( $ClientChatHandle[%sender] != 0 )
					alxStop( $ClientChatHandle[%sender] );
				$ClientChatHandle[%sender] = alxCreateSource( AudioChat, %wavFile );
				if (%pitch != 1.0)
					alxSourcef($ClientChatHandle[%sender], "AL_PITCH", %pitch);
				alxPlay( $ClientChatHandle[%sender] );
				%message = getSubStr(%message, 0, %wavStart);
				addMessageHudLine(%message);
			} else
				parent::clientCmdChatMessage(%sender, %voice, %pitch, %msgString, %a1, %a2, %a3, %a4, %a5, %a6, %a7, %a8, %a9, %a10);
				// Call back using the parent process to handle it if we don't have custom sound.
	}

	//Script setup stuff goes here, so it gets executed after certain needed functions are defined.
	function DispatchLaunchMode()
	{

		exec("scripts/VoicePackDX.gui");

		if(strstr($pref::video::resolution, "640")!=-1) {
			echo("Running at 640 mode...");
			%pos = "9 40";
		} else {
			%pos = "270 3";
		}

		echo("\c4Adding Button");

		new ShellBitmapButton(VoicePackManButton) {
			profile = "ShellButtonProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			position = %pos;
			extent = "150 32";
			minExtent = "32 38";
			visible = "1";
			hideCursor = "0";
			bypassHideCursor = "0";
			command = "Canvas.pushDialog(VoicePackDXManager);";
			helpTag = "0";
			text = "VOICEPACK OPTIONS";
			simpleStyle = "0";
		};
		GM_WarriorPane.add(VoicePackManButton);

		//Do Startup Stuff
		VPDXStartup();

		parent::DispatchLaunchMode();
	}
	function StartHostedGame()
	{
		RebuildVPDXMenus();
		parent::StartHostedGame();
	}

	function JoinGame(%address)
	{
		RebuildVPDXMenus();
		parent::JoinGame(%address);
	}
};
activatePackage(VoicePackDX);
